MODULE limhdf_2
   !!======================================================================
   !!                    ***  MODULE limhdf_2   ***
   !! LIM 2.0 ice model : horizontal diffusion of sea-ice quantities
   !!======================================================================
   !! History :  LIM  ! 2000-01 (UCL)  Original code
   !!            1.0  ! 2001-05 (G. Madec, R. Hordoir) opa norm
   !!            2.0  ! 2002-08 (C. Ethe)  F90, free form
   !!----------------------------------------------------------------------
#if defined key_lim2
   !!----------------------------------------------------------------------
   !!   'key_lim2'                                    LIM 2.0 sea-ice model
   !!----------------------------------------------------------------------
   !!   lim_hdf_2  : diffusion trend on sea-ice variable
   !!----------------------------------------------------------------------
   USE dom_oce          ! ocean domain
   USE ice_2            ! LIM-2 variables
   USE prtctl           ! Print control
   USE lbclnk           !
   USE lib_mpp          !
   USE in_out_manager   ! I/O manager

   IMPLICIT NONE
   PRIVATE

   PUBLIC   lim_hdf_2   ! called by lim_tra_2

   LOGICAL  ::   linit = .TRUE.              ! indictor of initialisation
   REAL(wp) ::   epsi04 = 1e-04              ! constant
   REAL(wp), DIMENSION(jpi,jpj) ::   zfact   ! metric term

   !! * Substitution 
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/LIM 3.3,  UCL-LOCEAN-IPSL (2010) 
   !! $Id$
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE lim_hdf_2( ptab )
      !!-------------------------------------------------------------------
      !!                  ***  ROUTINE lim_hdf_2  ***
      !!
      !! ** purpose :   Compute and add the diffusive trend on sea-ice variables
      !!
      !! ** method  :   Second order diffusive operator evaluated using a
      !!              Cranck-Nicholson time Scheme.
      !!
      !! ** Action  :   update ptab with the diffusive contribution
      !!-------------------------------------------------------------------
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj) ::   ptab   ! Field on which the diffusion is applied  
      !!
      INTEGER ::  ji, jj      ! dummy loop indices
      INTEGER ::  its, iter   ! temporary integers
      CHARACTER (len=55) :: charout
      REAL(wp) ::   zalfa, zrlxint, zconv, zeps   ! temporary scalars
      REAL(wp), DIMENSION(jpi,jpj) ::   zrlx, zflu, zflv, zdiv0, zdiv  ! temporary workspaces
      REAL(wp), DIMENSION(jpi,jpj) ::   ztab0                ! ???
      !!-------------------------------------------------------------------

      ! Initialisation
      ! ---------------   
      ! Time integration parameters
      zalfa = 0.5       ! =1.0/0.5/0.0 = implicit/Cranck-Nicholson/explicit
      its   = 100       ! Maximum number of iteration
      zeps  =  2. * epsi04

      ! Arrays initialization
      ztab0(:, : ) = ptab(:,:)
      zdiv0(:, 1 ) = 0.e0
      zdiv0(:,jpj) = 0.e0
      IF( .NOT.lk_vopt_loop ) THEN
         zflu (jpi,:) = 0.e0   
         zflv (jpi,:) = 0.e0
         zdiv0(1,  :) = 0.e0
         zdiv0(jpi,:) = 0.e0
      ENDIF

      ! Metric coefficient (compute at the first call and saved in
      IF( linit ) THEN
         DO jj = 2, jpjm1  
            DO ji = fs_2 , fs_jpim1   ! vector opt.
               zfact(ji,jj) = ( e2u(ji,jj) + e2u(ji-1,jj) + e1v(ji,jj) + e1v(ji,jj-1) ) / ( e1t(ji,jj) * e2t(ji,jj) )
            END DO
         END DO
         linit = .FALSE.
      ENDIF


      ! Sub-time step loop
      zconv = 1.e0
      iter  = 0

      !                                                   !======================!
      DO WHILE ( ( zconv > zeps ) .AND. (iter <= its) )   !  Sub-time step loop  !
         !                                                !======================!
         iter = iter + 1               ! incrementation of the sub-time step number

         DO jj = 1, jpjm1              ! diffusive fluxes in U- and V- direction
            DO ji = 1 , fs_jpim1   ! vector opt.
               zflu(ji,jj) = pahu(ji,jj) * e2u(ji,jj) / e1u(ji,jj) * ( ptab(ji+1,jj) - ptab(ji,jj) )
               zflv(ji,jj) = pahv(ji,jj) * e1v(ji,jj) / e2v(ji,jj) * ( ptab(ji,jj+1) - ptab(ji,jj) )
            END DO
         END DO
         DO jj= 2, jpjm1               ! diffusive trend : divergence of the fluxes
            DO ji = fs_2 , fs_jpim1   ! vector opt. 
               zdiv (ji,jj) = (  zflu(ji,jj) - zflu(ji-1,jj  )   &
                  &            + zflv(ji,jj) - zflv(ji  ,jj-1)  ) / ( e1t (ji,jj) * e2t (ji,jj) )
            END DO
         END DO
         !                             ! save the first evaluation of the diffusive trend in zdiv0
         IF( iter == 1 )   zdiv0(:,:) = zdiv(:,:)       

         DO jj = 2, jpjm1              ! XXXX iterative evaluation?????
            DO ji = fs_2 , fs_jpim1   ! vector opt.
               zrlxint = (   ztab0(ji,jj)    &
                  &       +  rdt_ice * (           zalfa   * ( zdiv(ji,jj) + zfact(ji,jj) * ptab(ji,jj) )   &
                  &                      + ( 1.0 - zalfa ) *   zdiv0(ji,jj) )  )                             & 
                  &    / ( 1.0 + zalfa * rdt_ice * zfact(ji,jj) )
               zrlx(ji,jj) = ptab(ji,jj) + om * ( zrlxint - ptab(ji,jj) )
            END DO
         END DO
         CALL lbc_lnk( zrlx, 'T', 1. )

         zconv = 0.e0                  ! convergence test
         DO jj = 2, jpjm1
            DO ji = 2, jpim1
               zconv = MAX( zconv, ABS( zrlx(ji,jj) - ptab(ji,jj) )  )
            END DO
         END DO
         IF( lk_mpp )   CALL mpp_max( zconv )   ! max over the global domain
         !
         ptab(:,:) = zrlx(:,:)         ! update value
         !                                         !=============================!
      END DO                                       !  end of sub-time step loop  !
      !                                            !=============================!

      IF(ln_ctl)   THEN
         zrlx(:,:) = ptab(:,:) - ztab0(:,:)
         WRITE(charout,FMT="(' lim_hdf  : zconv =',D23.16, ' iter =',I4,2X)") zconv, iter
         CALL prt_ctl( tab2d_1=zrlx, clinfo1=charout )
      ENDIF
      !
   END SUBROUTINE lim_hdf_2

#else
   !!----------------------------------------------------------------------
   !!   Default option          Dummy module       NO LIM 2.0 sea-ice model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE lim_hdf_2       ! Empty routine
   END SUBROUTINE lim_hdf_2
#endif

   !!======================================================================
END MODULE limhdf_2
